#  include <boost/units/systems/si.hpp>
using namespace boost::units;

#  include <blitz/array.h>
using namespace blitz;

typedef double real_t;

#  include "blitz-gnuplot.hpp"
#  include "adv_upstream.hpp"
#  include "adv_mpdata.hpp"
#  include "adv_leapfrog.hpp"

int main()
{
  // parameters (FIXME: should be taken from command-line options)
  int nx = 64, nt = 32;
  quantity<si::length, real_t> dx = 1 * si::metres;
  quantity<si::time, real_t> dt = 1 * si::seconds;
  quantity<si::velocity, real_t> u = .3 * si::metres / si::seconds;
  quantity<si::dimensionless, real_t> C = u * dt / dx;
  string advscheme = "mpdata";

  // advection sheme "allocation"
  adv<si::dimensionless, real_t> *advsch, *fllbck = NULL;
  if (advscheme == "upstream") 
    advsch = new adv_upstream<si::dimensionless, real_t>();
  else if (advscheme == "leapfrog")
  {
    advsch = new adv_leapfrog<si::dimensionless, real_t>();
    fllbck = new adv_upstream<si::dimensionless, real_t>();
  }
  else if (advscheme == "mpdata")
    advsch = new adv_mpdata<si::dimensionless, real_t>();
  else assert(false);

  // memory allocation 
  assert(advsch->stencil_extent() == 3); // FIXME: support for other values
  Array<quantity<si::dimensionless, real_t>, 1> **psi; 
  psi = new Array<quantity<si::dimensionless, real_t>, 1>*[advsch->time_levels()]; // FIXME: no deallocation yet!
  for (int n=0; n < advsch->time_levels(); ++n) 
    psi[n] = new Array<quantity<si::dimensionless, real_t>, 1>(Range(-1,nx)); // FIXME: ditto

  // initial condition 
  (*psi[0])(Range(nx/4, nx/2)) = 1;

  // integration loop
  for (int t = 0; t < nt; ++t)
  {
    assert(advsch->time_levels() <= 3); // FIXME: support for other values
    bool fallback = (t == 0 && advsch->time_levels() == 3);
    adv<si::dimensionless, real_t> *a = fallback ? fllbck : advsch;
    int n = a->time_levels() - 2;

    // output
    cout << *psi[n] << endl << endl;

    for (int s = 1; s <= a->num_steps(); ++s)
    {
      // periodic boundary
      (*psi[n])(Range(-1)) = (*psi[n])(Range(nx-1));
      (*psi[n])(Range(nx)) = (*psi[n])(Range(0));

      // copying data from psi[0] -> psi[n+1]
      *psi[n+1] = *psi[0];

      // advection operator
      a->op(psi, Range(0, nx-1), n, C, s);

      // swapping arrays
      if (!fallback) switch (advsch->time_levels())
      {   
        case 2: cycleArrays(*psi[n], *psi[n+1]); break;
        case 3: cycleArrays(*psi[n-1], *psi[n], *psi[n+1]); break;
        default: assert(false);
      }   
    }
  }
}
